#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <fstream>
#include <iomanip>

using namespace std;

#include "File.h"

void readMarkerFile3D(CSet &SET){
	
	ifstream inFile(SET.filename);
	if(inFile.bad() || !inFile.is_open()){
		printf("Problems on file opening");
		return;
	}
	
	string names;
	int fr=0,cam=0,oldcam=1,oldfr=1;
	double R=0.0,fps=10.0,timet=0.0;
	CData object2d;
	CData line3d;
	CFrame frame;
	CCamera camera;
	
	for (int ii=0;ii<10;ii++){
	    inFile >> names;
	}
	
	while (!inFile.eof()){
	    inFile >> fr;
	    inFile >> cam;
	    inFile >> object2d.x;
	    inFile >> object2d.y;
	    inFile >> line3d.x;
	    inFile >> line3d.y;
	    inFile >> line3d.z;
	    inFile >> line3d.x2;
	    inFile >> line3d.y2;
	    inFile >> line3d.z2;
	    object2d.timet=timet;
	    line3d.timet=timet;
					      
	    if (cam>oldcam){
		frame.cam.push_back(camera);
		camera.objects2d.clear();
		camera.objects3d.clear();
		oldcam=cam;
	    }
	    if (fr>oldfr){
		frame.cam.push_back(camera);
		camera.objects2d.clear();
		camera.objects3d.clear();
		if (frame.cam.size()==1){
		  frame.cam.push_back(camera);
		}
		frame.timet=timet;
		frame.fn=oldfr;
		timet+=1/fps;
		SET.frames.push_back(frame);
		frame.cam.clear();
		oldfr=fr;
		oldcam=1;
	    }	
	    
	    camera.objects2d.push_back(object2d);
	    camera.objects3d.push_back(line3d);
	    
	}
	
	frame.cam.push_back(camera);
	frame.timet=timet;
	timet+=1/fps;
	camera.objects2d.clear();
	camera.objects3d.clear();
	SET.frames.push_back(frame);
	frame.cam.clear();
	
	for (int fr=0;fr<SET.frames.size();fr++){
	    for (int ncam=0;ncam<SET.nCam;ncam++){
		for (int p=0;p<SET.frames[fr].cam[ncam].objects2d.size();p++){
		    SET.frames[fr].cam[ncam].objects2d[p].timet=SET.frames[fr].timet;
		}
	    }
	}
		
	inFile.close();
  
}


void readMarkerFile(CSet &SET){
	
	ifstream inFile(SET.filename);
	if(inFile.bad() || !inFile.is_open()){
		printf("Problems on file opening");
		return;
	}
	
	string names;
	int fr=0,cam=0,oldcam=1,oldfr=0;
	double R=0.0,fps=120.0,timet=0.0;
	CData object2d;
	CFrame frame;
	CCamera camera;
	
	for (int ii=0;ii<5;ii++){
	    inFile >> names;
	}
	
	while (!inFile.eof()){
	    inFile >> fr;
	    inFile >> cam;
	    inFile >> object2d.x;
	    inFile >> object2d.y;
	    inFile >> R;
	    if (cam>oldcam){
		frame.cam.push_back(camera);
		camera.objects2d.clear();
		oldcam=cam;
	    }
	    if (fr>oldfr){
		frame.cam.push_back(camera);
		frame.timet=timet;
		timet+=1/fps;
		camera.objects2d.clear();
		SET.frames.push_back(frame);
		frame.cam.clear();
		oldfr=fr;
	    }	
	    camera.objects2d.push_back(object2d);
	}
	
	inFile.close();
  
}

void readFileWithRectangle(CSet &SET, const int ncam){
	
	ifstream inFile(SET.filename);
	if(inFile.bad() || !inFile.is_open()){
		printf("Problems on file opening");
		return;
	}

	int fn=-1, id =-1;
	float x_pix=-1,y_pix=-1,z_pix=-1;
	float _x=0.0,_y=0.0,_z=0.0,vx=0.0,vy=0.0,vz=0.0,timet=0.0,groupid=0,width=0,height=0;
	CData object2d;
	CData object3dcam;
	CFrame frame;
	CCamera camera;
	SET.usedColors=-1;

	char line[c_strSize];
	memset(line,0, c_strSize);

	
	      bool first=true;
	      int FrameNum=0;
	      
	      do{
		      inFile.getline(line, c_strSize); 

			      if(line[0] != '/' && line[0] != ' ' && line[0] != 0 && line[0]!= '?' && line[0] != '\r'){
				      int linesize = strlen(line);
				      if(line[0] == '#'){ 
					      if (line[1]=='F'){
						      sscanf(line, "#F%d", &fn);
					      }else if (line[1]=='C'){
						      sscanf(line, "#CIL%d", &fn);
					      }else{
						      sscanf(line, "#%d", &fn);
					      }
					      //particle.fn = fn;
					      if(!first){
							if (ncam==0){
							  frame.cam.push_back(camera);
							  SET.frames.push_back(frame);
							  frame.cam.clear();
							  camera.objects2d.clear();
							  camera.objects3d.clear();
							}else{
							  SET.frames[FrameNum].cam.push_back(camera);
							  camera.objects2d.clear();
							  camera.objects3d.clear();
							}
							FrameNum++;
					      }				
					      first=false;
					      frame.fn = fn;
				      }else if (fn>=0) {
					      sscanf(line, "%d %f %f %f %f %f %f %f %f %f %f %f %f",&id, &x_pix, &y_pix, &_x, &_y, &_z, &vx, &vy, &vz, &width, &height, &groupid, &timet);
					      object2d.id = id;
					      object2d.x = x_pix;
					      object2d.y = y_pix;
					      object2d.z = -1;
					      object3dcam.x = _x;
					      object3dcam.y = _y;
					      object3dcam.z = _z;
					      object2d.vx = vx;
					      object2d.vy = vy;
					      object2d.vz = 0;
					      object2d.timet=timet;
					      object2d.created=false;
					      object2d.autoclass=id;
					      if (id>SET.usedColors){
						  SET.usedColors=id;
					      }
					      object2d.oldclass=-1;
					      object2d.predicted=false;
					      object2d.width=int(width);
					      object2d.height=int(height);
					      object2d.fn=SET.frames.size();
					      frame.timet=timet;
					      camera.objects2d.push_back(object2d);
					      camera.objects3d.push_back(object3dcam);
				      }		
			      }
		      
	      }while(!inFile.eof());
	      
	      if(fn>=0){ 
		  if (ncam==0){
		      frame.cam.push_back(camera);
		      SET.frames.push_back(frame);
		  }else{
		      SET.frames[FrameNum].cam.push_back(camera);		    
		  }
	      }
	      
	      inFile.close();		
	
}


bool writeFileWithRectangle(CSet &SET){
	

	for (int ncam=0;ncam<SET.nCam;ncam++){
		char StringCam[256];
		sprintf(StringCam,"%d",ncam+1);
		strcpy(SET.ofilename,SET.MAIN_DIR);
		strcat(SET.ofilename,"result_MV_");
		strcat(SET.ofilename,SET.FILE);
		strcat(SET.ofilename,"_");
		strcat(SET.ofilename,StringCam);
		strcat(SET.ofilename,".txt");
		
		ofstream outFile(SET.ofilename); 

		if(!outFile.is_open()){
			cout<< "No Output File";
			return false;
			
		}
		for (int f=0;f<SET.frames.size();f++){
			outFile << "#F" << f<< endl;
			for(int p=0;p<SET.frames[f].cam[ncam].objects2d.size();p++){
				outFile << SET.frames[f].cam[ncam].objects2d[p].autoclass << " " 
				<<SET.frames[f].cam[ncam].objects2d[p].x <<  " " 
				<<SET.frames[f].cam[ncam].objects2d[p].y <<  " " 
				<<SET.frames[f].cam[ncam].objects3d[p].x << " " 
				<<SET.frames[f].cam[ncam].objects3d[p].y << " " 
				<<SET.frames[f].cam[ncam].objects3d[p].z <<" " 
				<<SET.frames[f].cam[ncam].objects3d[p].vx << " "  
				<<SET.frames[f].cam[ncam].objects3d[p].vy << " "
				<<SET.frames[f].cam[ncam].objects3d[p].vz <<" "  
				<<SET.frames[f].cam[ncam].objects2d[p].width <<" "  
				<<SET.frames[f].cam[ncam].objects2d[p].height <<" "  
				<<" 0 " // group.id <<"  " 
				<<SET.frames[f].cam[ncam].objects2d[p].timet << " " <<endl;
			}
			outFile <<  endl;
		}
		outFile.close();
	}
		
	return true;
}


bool writeFileMarkers(CSet &SET){
	

	for (int ncam=0;ncam<SET.nCam;ncam++){
		char StringCam[256];
		sprintf(StringCam,"%d",ncam+1);
		strcpy(SET.ofilename,SET.MAIN_DIR);
		strcat(SET.ofilename,"result_MV_");
		strcat(SET.ofilename,SET.FILE);
		strcat(SET.ofilename,"_");
		strcat(SET.ofilename,StringCam);
		strcat(SET.ofilename,".txt");
		
		ofstream outFile(SET.ofilename); 

		if(!outFile.is_open()){
			cout<< "No Output File";
			return false;
			
		}
		for (int f=0;f<SET.frames.size();f++){
			outFile << "#F" << SET.frames[f].fn<< endl;
			for(int p=0;p<SET.frames[f].cam[ncam].objects2d.size();p++){
				outFile << SET.frames[f].cam[ncam].objects2d[p].autoclass << " " 
				<<SET.frames[f].cam[ncam].objects2d[p].x <<  " " 
				<<SET.frames[f].cam[ncam].objects2d[p].y <<  " " 
				<<SET.frames[f].cam[ncam].objects3d[p].x << " " 
				<<SET.frames[f].cam[ncam].objects3d[p].y << " " 
				<<SET.frames[f].cam[ncam].objects3d[p].z <<" " 
				<<SET.frames[f].cam[ncam].objects3d[p].vx << " "  
				<<SET.frames[f].cam[ncam].objects3d[p].vy << " "
				<<SET.frames[f].cam[ncam].objects3d[p].vz <<" "  
				<<SET.frames[f].cam[ncam].objects2d[p].width <<" "  
				<<SET.frames[f].cam[ncam].objects2d[p].height <<" "  
				<<" 0 " // group.id <<"  " 
				<<SET.frames[f].cam[ncam].objects2d[p].timet << " " <<endl;
			}
			outFile <<  endl;
		}
	}
		
	return true;
}
