

#include "File.h"
#include "RW4DW.h"
#define DEBUG


const int rowcollim=100000;//100000000;
int MAX_RECOVER;

vector <int> NOTERASED;

using namespace std;


/* INPUTS: 

1. Main directory
2. Input file name
3. Directory where dw_solver is found
4. Number of cameras or views V (page 4)
5. Number of objects to track N_obj (page 5)
6. Maximum number of frames to recover
7. Maximum distance in image coordinates V_max^2D (page 3)
8. Maximum distance in real world coordinates V_max^3D (page 3)
9. Bias that favors matching detections in consecutive frames B_j (page 3)

*/

int ParseInputs(char* argv[], CSet& SET){

	SET.filename[0]='\0';
	SET.ofilename[0]='\0';
	SET.costfile[0]='\0';

	SET.MAIN_DIR=argv[1];
	SET.FILE=argv[2];
	SET.DWSOLVER_DIR=argv[3];
	
	SET.nObj=atoi(argv[5]);
	
	strcpy(SET.costfile,SET.MAIN_DIR);
	strcat(SET.costfile,"costs.txt");
	SET.costs.open(SET.costfile);
	
	SET.RecoverFrames=atoi(argv[6]); 
	SET.maxspeed=atof(argv[7]); //per second
	SET.maxspeed3D=atof(argv[8]); //per second
	
	SET.base_jump=0.3;
	
	
	cout << "Base jump " << SET.base_jump << endl;
			
	
	for (int ncam=0;ncam<SET.nCam;ncam++){
	  
	      char StringCam[256];
	      sprintf(StringCam,"%d",ncam+1);
	      strcpy(SET.filename,SET.MAIN_DIR);
	      strcat(SET.filename,SET.FILE);
	      strcat(SET.filename,"_");
	      strcat(SET.filename,StringCam);
	      strcat(SET.filename,".txt");
	      
	      cout << SET.filename << endl;
	      
	      readFileWithRectangle(SET,ncam);
	      
	}
	
	return atoi(argv[9]);
	  
}


int main(int argc, char* argv[]){


	CSet SET(atoi(argv[4]));
	clock_t startC, endC;
	
	int Nframes=ParseInputs(argv,SET);
	
	double tdiff=0.0;
	for (int fr=0;fr<SET.frames.size()-1;fr++){
		if ((SET.frames[fr+1].timet-SET.frames[fr].timet)<0)
			tdiff=tdiff+SET.frames[fr+1].timet-SET.frames[fr].timet+60;
		else
			tdiff=tdiff+SET.frames[fr+1].timet-SET.frames[fr].timet;
	}
	tdiff=tdiff/(SET.frames.size()-1);
	
	int usedC=0;
	int fr_ini_old=-1,fr_end_old=-1;
	
	cout << "Maximum recovery frames:  " << SET.RecoverFrames << endl;
	int iter=1,changes=1;

			
	cout << "Number of total objects:  " << SET.nObj << endl;
	cout << "Number of frames:  " << SET.frames.size() << endl;

	vector<int> Ncols;
	int totalObj=0,iternum=0,count=0;
	bool repeat=true;
	
	
	
	for (int fr_ini=0,fr_end=0;fr_end<SET.frames.size()-1;fr_ini=fr_end-SET.RecoverFrames){//SET.frames.size()-1
		
		if (fr_ini_old>=fr_ini){
		    fr_ini=fr_ini_old+SET.RecoverFrames/2;
		}
		if (fr_ini_old>-1){
		    SET.Erase3DPossibilities(fr_ini_old,fr_ini);
		}
	  
		if (fr_ini>0){
		      for (int fr_erase=fr_ini+1;fr_erase<=fr_end;fr_erase++){
			    for (int ncam=0;ncam<SET.nCam;ncam++){
				  for (int p=0;p<SET.frames[fr_erase].cam[ncam].objects2d.size();p++){
					SET.frames[fr_erase].cam[ncam].objects2d[p].autoclass=-1;
					SET.frames[fr_erase].cam[ncam].cam2.clear();
				  }
			    }
		      }
		}
		
		
		fr_end=fr_ini+Nframes;
		if ((SET.frames.size()-1)<fr_end )
			fr_end=SET.frames.size()-1;
		
		if (fr_ini>=fr_end){
		    break;
		}
		
		
		totalObj=0;
	
		do {
			totalObj=0;
			for (int fro=fr_ini;fro<=fr_end;fro++){
			      for (int ncam=0;ncam<SET.nCam;ncam++){ 
				    totalObj=totalObj+SET.frames[fro].cam[ncam].objects2d.size();
			      }
			}
			fr_end--;
		}while (totalObj>100);
		    
		fr_end++;
		

		cout << "Create the input file of the LP problem... with NObj = " << SET.nObj << " from FR " << fr_ini << " to FR " << fr_end << endl;	
		
		
		Ncols=CreateDWproblem(SET,fr_ini,fr_end,tdiff,0);
		
		cout << Ncols[Ncols.size()-1] << endl;
		
		cout << "Solve LP problem... " << endl;
		
		strcpy(SET.filename,SET.MAIN_DIR);
		strcat(SET.filename,"master.cplex");
		strcpy(SET.ofilename,SET.MAIN_DIR);
		strcat(SET.ofilename,"integerized_zeros"); 
		if( access( SET.ofilename, F_OK ) == -1 ){
		    strcpy(SET.filename,SET.MAIN_DIR);
		    strcat(SET.filename,"master.cplex");
		    strcpy(SET.ofilename,SET.MAIN_DIR);
		    strcat(SET.ofilename,"relaxed_solution");
		}
				
		
		repeat=true;iternum=0;

		
		char comd[256];
		comd[0]='\0';
		strcpy(comd,"cd ");
		strcat(comd,SET.MAIN_DIR);
		strcat(comd,"; ");
		strcat(comd,SET.DWSOLVER_DIR);
		strcat(comd," -g guide_file"); 
		cout << comd << endl;
		int resp=system(comd);

		cout << "Read the output CPLEX file and copy results to SET... " << endl;
		
		ReadCPLEXSolution(SET, fr_ini, fr_end,tdiff,Ncols);
	
	
		fr_ini_old=fr_ini;
		fr_end_old=fr_end;
		
		for (int crfr=fr_ini;crfr<=fr_end;crfr++){
		    for (int ncam=0;ncam<SET.nCam;ncam++){ 
			for (int p=0;p<SET.frames[crfr].cam[ncam].objects2d.size();p++){
			    SET.frames[crfr].cam[ncam].objects2d[p].created=false;
			}
		    }
		}
		
		
		writeFileWithRectangle(SET);
		
	
	}


	
	//writeFileWithRectangle(SET);
	

	SET.costs.close();
	SET.eraseSet();
	
	return 0;
}

/* eof */
